/**
 * Copyright 2005 The Apache Software Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.apache.nutch.parse.rss;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.List;
import java.util.Properties;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.apache.nutch.parse.Outlink;
import org.apache.nutch.parse.Parse;
import org.apache.nutch.parse.ParseData;
import org.apache.nutch.parse.ParseException;
import org.apache.nutch.parse.ParseImpl;
import org.apache.nutch.parse.Parser;
import org.apache.nutch.protocol.Content;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Node;
import org.dom4j.io.DocumentResult;
import org.dom4j.io.DocumentSource;
import org.dom4j.io.SAXReader;

/**
 * 
 * A RSS Parser
 * 
 * created on 28.03.2005
 * 
 * @author sg
 * @version $Revision: 1.2 $
 *  
 */
public class RSSParser implements Parser {
    public Parse getParse(Content content) throws ParseException {
        try {

            Properties metadata = content.getMetadata();
            if (metadata == null) {
                metadata = new Properties();
            }

            Item[] items = extractContent(content.getContent());
            StringBuffer text = new StringBuffer();
            String title = "";
            Outlink[] outlinks = new Outlink[items.length];

            for (int i = 0; i < items.length; i++) {

                text.append(items[i].getTitle());
                text.append(" ");
                text.append(items[i].getDescription());
                text.append(" ");
                outlinks[i] = new Outlink(items[i].getUrl(), items[i]
                        .getTitle());

            }

            return new ParseImpl(text.toString(), new ParseData(title,
                    outlinks, metadata));
        } catch (Exception e) {
            throw new ParseException(e);
        }
    }

    private Item[] extractContent(byte[] bs) throws Exception {

        Document sourceXml = getDocument(bs);
        Document styleDocument = transformDocument(sourceXml);
        List nodeTitles = styleDocument.selectNodes("rss/channel/item/title");
        List nodeDescriptions = styleDocument
                .selectNodes("rss/channel/item/description");
        List nodeItemUrls = styleDocument.selectNodes("rss/channel/item/link");

        Item[] items = new Item[nodeTitles.size()];
        for (int i = 0; i < nodeTitles.size(); i++) {
            String title = ((Node) nodeTitles.get(i)).getStringValue();
            String description = ((Node) nodeDescriptions.get(i))
                    .getStringValue();
            String itemUrl = ((Node) nodeItemUrls.get(i)).getStringValue();
            items[i] = new Item(title, description, itemUrl);
        }

        return items;

    }

    /**
     * transforms any RSS version to a standart xml document
     * 
     * @param document
     * @return transformedDoc
     * @throws Exception
     */
    private static Document transformDocument(Document document)
            throws Exception {

        // load the transformer using JAXP
        TransformerFactory factory = TransformerFactory.newInstance();
        InputStream xslInputStream = RSSParser.class
                .getResourceAsStream("/rss.xslt");
        StreamSource xsltSource = new StreamSource(xslInputStream);
        Transformer transformer = factory.newTransformer(xsltSource);

        // now lets style the given document
        DocumentSource documentSource = new DocumentSource(document);
        DocumentResult result = new DocumentResult();
        transformer.transform(documentSource, result);

        // return the transformed document
        Document transformedDoc = result.getDocument();
        return transformedDoc;
    }

    /**
     * generates a Document from given byte[] argument
     * 
     * @param bs
     * @return Xml document
     * @throws DocumentException
     */
    private Document getDocument(byte[] bs) throws DocumentException {
        SAXReader reader = new SAXReader();
        Document document = reader.read(new ByteArrayInputStream(bs));
        return document;
    }
}